@@ -35,6 +35,7 @@ from .utils.graph import Graph
 from .utils.analysis import Timer
 from .utils.portreference import PortReference
 from .utils.progresstracker import ProgressTracker
+from .utils.logger import LoggerManager
 
 from .solvers import SSPRK22, SteadyState
 
@@ -153,8 +154,10 @@ class Simulation:
         get attributes and access to intermediate evaluation stages
     logger : logging.Logger
         global simulation logger
-    _blocks_dyn : list[Block]
-        list of blocks with internal ´Solver´ instances (stateful) 
+    _blocks_dyn : set[Block]
+        blocks with internal ´Solver´ instances (stateful) 
+    _blocks_evt : set[Block]
+        blocks with internal events (discrete time, eventful) 
     _active : bool
         flag for setting the simulation as active, used for interrupts
     """
@@ -175,9 +178,9 @@ class Simulation:
         ):
 
         #system definition
-        self.blocks      = []
-        self.connections = []
-        self.events      = []
+        self.blocks      = set()
+        self.connections = set()
+        self.events      = set()
 
         #simulation timestep and bounds
         self.dt     = dt
@@ -212,7 +215,10 @@ class Simulation:
         self.time = 0.0
 
         #collection of blocks with internal ODE solvers
-        self._blocks_dyn = []
+        self._blocks_dyn = set()
+
+        #collection of blocks with internal events
+        self._blocks_evt = set()
 
         #flag for setting the simulation active
         self._active = True
@@ -223,12 +229,12 @@ class Simulation:
         #prepare and add blocks (including internal events)
         if blocks is not None:
             for block in blocks:
-                self.add_block(block)
+                self.add_block(block, _defer_graph=True)
 
         #check and add connections
         if connections is not None:
             for connection in connections:
-                self.add_connection(connection)
+                self.add_connection(connection, _defer_graph=True)
 
         #check and add events
         if events is not None:
@@ -304,47 +310,38 @@ class Simulation:
     # logger methods --------------------------------------------------------------
 
     def _initialize_logger(self):
-        """
-        setup and configure logging
+        """Setup and configure logging using the centralized LoggerManager.
+
+        The logger is configured based on the 'log' parameter which can be:
+        - True: logging enabled to stdout
+        - False: logging disabled
+        - str: logging enabled to file at specified path
         """
 
-        #initialize the logger
-        self.logger = logging.Logger("PathSim_Simulation_Logger")
+        #get logger manager singleton
+        logger_mgr = LoggerManager()
 
-        #capture warnings from the 'warnings' module
-        logging.captureWarnings(True)
+        #configure based on log parameter
+        if self.log:
+            #determine output destination
+            output = self.log if isinstance(self.log, str) else None
 
-        #check if logging is enabled
-        if self.log:    
+            #configure logger manager with short timestamp
+            logger_mgr.configure(
+                enabled=True,
+                output=output,
+                level=logging.INFO,
+                date_format='%H:%M:%S'
+            )
+        else:
+            #disable logging
+            logger_mgr.configure(enabled=False)
 
-            #if a filename for logging is specified
-            if isinstance(self.log, str):
-                handler = logging.FileHandler(self.log) 
-            else:
-                handler = logging.StreamHandler()
+        #get logger for simulation
+        self.logger = logger_mgr.get_logger("simulation")
 
-            #logging format
-            handler.setFormatter(
-                logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")   
-                )
-
-            self.logger.addHandler(handler)
-            self.logger.setLevel(logging.INFO)
-
-            self._logger_info(f"LOGGING (log: {self.log})")
-
-
-    def _logger_info(self, message):
-        if self.log: self.logger.info(message)
-
-
-    def _logger_error(self, message, Error=None):
-        if self.log: self.logger.error(message)
-        if Error is not None: raise Error(message)
-
-
-    def _logger_warning(self, message):
-        if self.log: self.logger.warning(message)
+        #log initialization
+        self.logger.info(f"LOGGING (log: {self.log})")
 
 
     # visualization ---------------------------------------------------------------
@@ -535,7 +532,7 @@ class Simulation:
 
     # adding system components ----------------------------------------------------
 
-    def add_block(self, block):
+    def add_block(self, block, _defer_graph=False):
         """Adds a new block to the simulation, initializes its local solver 
         instance and collects internal events of the new block. 
 
@@ -545,40 +542,36 @@ class Simulation:
         ----------
         block : Block 
             block to add to the simulation
+        _defer_graph : bool
+            flag for defering graph construction to a later stage
         """
 
         #check if block already in block list
         if block in self.blocks:
             _msg = f"block {block} already part of simulation"
-            self._logger_error(_msg, ValueError)
+            self.logger.error(_msg)
+            raise ValueError(_msg)
 
         #initialize numerical integrator of block with parent
         block.set_solver(self.Solver, self.engine, **self.solver_kwargs)
 
         #add to dynamic list if solver was initialized
         if block.engine and block not in self._blocks_dyn:
-            self._blocks_dyn.append(block)
+            self._blocks_dyn.add(block)
+
+        #add to eventful list if internal events
+        if block.events:
+            self._blocks_evt.add(block)
 
         #add block to global blocklist
-        self.blocks.append(block)
-
-        #logging message
-        self._logger_info(
-            "BLOCK (type: {}, dynamic: {}, events: {})".format(
-                block.__class__.__name__, bool(block.engine), len(block.events)
-                )
-            )
-
-        #add events of block to global event list
-        for event in block.events:
-            self.add_event(event)
+        self.blocks.add(block)
 
         #if graph already exists, it needs to be rebuilt
-        if self.graph:
+        if not _defer_graph and self.graph:
             self._assemble_graph()
 
 
-    def add_connection(self, connection):
+    def add_connection(self, connection, _defer_graph=False):
         """Adds a new connection to the simulaiton and checks if 
         the new connection overwrites any existing connections.
 
@@ -588,24 +581,21 @@ class Simulation:
         ----------
         connection : Connection
             connection to add to the simulation
+        _defer_graph : bool
+            flag for defering graph construction to a later stage
         """
 
         #check if connection already in connection list
         if connection in self.connections:
             _msg = f"{connection} already part of simulation"
-            self._logger_error(_msg, ValueError)
-
-        #check if connection overwrites existing connections
-        for conn in self.connections:
-            if connection.overwrites(conn):
-                _msg = f"{connection} overwrites {conn}"
-                self._logger_error(_msg, ValueError)
+            self.logger.error(_msg)
+            raise ValueError(_msg)
 
         #add connection to global connection list
-        self.connections.append(connection)
+        self.connections.add(connection)
 
         #if graph already exists, it needs to be rebuilt
-        if self.graph:
+        if not _defer_graph and self.graph:
             self._assemble_graph()
 
 
@@ -623,10 +613,11 @@ class Simulation:
         #check if event already in event list
         if event in self.events:
             _msg = f"{event} already part of simulation"
-            self._logger_error(_msg, ValueError)
+            self.logger.error(_msg)
+            raise ValueError(_msg)
 
         #add event to global event list
-        self.events.append(event)
+        self.events.add(event)
 
 
     # system assembly -------------------------------------------------------------
@@ -646,7 +637,17 @@ class Simulation:
                 ConnectionBooster(conn) for conn in self.graph.loop_closing_connections()
             ]
 
-        self._logger_info(
+        #log block summary
+        num_dynamic = len(self._blocks_dyn)
+        num_static = len(self.blocks) - num_dynamic
+        num_eventful = len(self._blocks_evt)
+        self.logger.info(
+            f"BLOCKS (total: {len(self.blocks)}, dynamic: {num_dynamic}, "
+            f"static: {num_static}, eventful: {num_eventful})"
+            )
+
+        #log graph info
+        self.logger.info(
             "GRAPH (nodes: {}, edges: {}, alg. depth: {}, loop depth: {}, runtime: {})".format(
                 *self.graph.size, *self.graph.depth, T
                 )
@@ -657,22 +658,22 @@ class Simulation:
 
     def _check_blocks_are_managed(self):
         """Check whether the blocks that are part of the connections are 
-        in the simulation block list ('self.blocks') and therefore managed 
+        in the simulation block set ('self.blocks') and therefore managed 
         by the simulation.
 
         If not, there will be a warning in the logging.            
         """
-        #collect blocks from connections
-        conn_blocks = []
-        for conn in self.connections:
-            conn_blocks.extend(conn.get_blocks())
 
-        #iterate set of blocks from connections (unique)
-        for blk in set(conn_blocks):
-            if blk not in self.blocks:
-                self._logger_warning(
-                    f"{blk} in 'connections' but not in 'blocks'!"
-                    )
+        # Collect connection blocks
+        conn_blocks = set()
+        for conn in self.connections:
+            conn_blocks.update(conn.get_blocks())
+
+        # Check subset actively managed
+        if not conn_blocks.issubset(self.blocks):
+            self.logger.warning(
+                f"{blk} in 'connections' but not in 'blocks'!"
+                )
 
 
     # solver management -----------------------------------------------------------
@@ -703,20 +704,20 @@ class Simulation:
         self.engine = self.Solver()
 
         #iterate all blocks and set integration engines with tolerances
-        self._blocks_dyn = []
+        self._blocks_dyn = set()
         for block in self.blocks:
             block.set_solver(self.Solver, self.engine, **self.solver_kwargs)
             
             #add dynamic blocks to list
             if block.engine:
-                self._blocks_dyn.append(block)
-        
+                self._blocks_dyn.add(block)
+
         #logging message
-        self._logger_info(
+        self.logger.info(
             "SOLVER (dyn. blocks: {}) -> {} (adaptive: {}, explicit: {})".format(
                 len(self._blocks_dyn),
                 self.engine,
-                self.engine.is_adaptive, 
+                self.engine.is_adaptive,
                 self.engine.is_explicit
                 )
             )
@@ -743,7 +744,7 @@ class Simulation:
             simulation time for reset
         """
 
-        self._logger_info(f"RESET (time: {time})")
+        self.logger.info(f"RESET (time: {time})")
 
         #set active again
         self._active = True
@@ -788,7 +789,7 @@ class Simulation:
             for block in self.blocks:
                 block.linearize(self.time)
 
-        self._logger_info(f"LINEARIZED (runtime: {T})")
+        self.logger.info(f"LINEARIZED (runtime: {T})")
 
 
     def delinearize(self):
@@ -796,11 +797,22 @@ class Simulation:
         for block in self.blocks: 
             block.delinearize()
 
-        self._logger_info("DELINEARIZED")
+        self.logger.info("DELINEARIZED")
 
 
     # event system helpers --------------------------------------------------------
 
+    def _get_active_events(self):
+        """Helper method to collect all active events"""
+        events = []
+        for event in self.events:
+            if event: events.append(event)
+        for block in self._blocks_evt:
+            for event in block.events:
+                if event: events.append(event)
+        return events
+
+
     def _estimate_events(self, t):
         """Estimate the time until the next.
 
@@ -816,40 +828,24 @@ class Simulation:
         """
 
         dt_evt_min = None
-        for event in self.events:
 
-            #skip inactive events
-            if not event: continue
+        #check external events
+        for event in self._get_active_events():
 
             #get the estimate
             dt_evt = event.estimate(self.time)
+
+            #no estimate available
+            if dt_evt is None: continue
             
-            #check if estimate available and smaller than min
-            if dt_evt_min is None or (dt_evt is not None and dt_evt < dt_evt_min):
+            #smaller than min
+            if dt_evt_min is None or dt_evt < dt_evt_min:
                 dt_evt_min = dt_evt
 
         #return time until next event or None
         return dt_evt_min
 
 
-    def _buffer_events(self, t):
-        """Buffer states for event monitoring before the timestep 
-        is taken. 
-
-        This is required to set reference for event monitoring and 
-        backtracking for root finding.
-
-        Parameters
-        ----------
-        t : float 
-            evaluation time for buffering
-        """
-
-        #buffer states for event detection (with timestamp)
-        for event in self.events:
-            if event: event.buffer(t)
-
-
     def _detected_events(self, t):
         """Check for possible (active) events and return them chronologically, 
         sorted by their timestep ratios (closest to the initial point in time).
@@ -867,10 +863,7 @@ class Simulation:
 
         #iterate all event managers
         detected_events = []
-        for event in self.events:
-
-            #skip inactive events
-            if not event: continue
+        for event in self._get_active_events():
             
             #check if an event is detected
             detected, close, ratio = event.detect(t)
@@ -985,12 +978,11 @@ class Simulation:
                 return
 
         #not converged -> error
-        self._logger_error(
-            "algebraic loop not converged (iters: {}, err: {})".format(
-                self.iterations_max, max_err
-                ), 
-            RuntimeError
+        _msg = "algebraic loop not converged (iters: {}, err: {})".format(
+            self.iterations_max, max_err
             )
+        self.logger.error(_msg)
+        raise RuntimeError(_msg)
 
 
     def _solve(self, t, dt):
@@ -1097,7 +1089,7 @@ class Simulation:
         self._set_solver(SteadyState)
 
         #log message begin of steady state solver
-        self._logger_info(f"STEADYSTATE -> STARTING (reset: {reset})")
+        self.logger.info(f"STEADYSTATE -> STARTING (reset: {reset})")
 
         #solve for steady state at current time
         with Timer(verbose=False) as T:
@@ -1105,17 +1097,16 @@ class Simulation:
 
         #catch non convergence
         if not success:
-            self._logger_error(
-                "STEADYSTATE -> FINISHED (success: {}, evals: {}, iters: {}, runtime: {})".format(
-                    success, evals, iters, T), 
-                RuntimeError
-                )
+            _msg = "STEADYSTATE -> FINISHED (success: {}, evals: {}, iters: {}, runtime: {})".format(
+                success, evals, iters, T)
+            self.logger.error(_msg)
+            raise RuntimeError(_msg)
 
         #sample result
-        self._sample(self.time)
+        self._sample(self.time, self.dt)
 
-        #log message 
-        self._logger_info(
+        #log message
+        self.logger.info(
             "STEADYSTATE -> FINISHED (success: {}, evals: {}, iters: {}, runtime: {})".format(
                 success, evals, iters, T)
             )
@@ -1126,21 +1117,29 @@ class Simulation:
 
     # timestepping helpers --------------------------------------------------------
 
-    def _revert(self):
+    def _revert(self, t):
         """Revert simulation state to previous timestep for adaptive solvers 
         when local truncation error is too large and timestep has to be 
-        retaken with smaller timestep. 
+        retaken with smaller timestep.
+
+        Parameters
+        ----------
+        t : float
+            evaluation time for simulation revert 
         """
 
-        #revert dummy engine (for history)
+        #revert dummy engine (for history, allways)
         self.engine.revert()
 
         #revert block states
         for block in self._blocks_dyn:
             if block: block.revert()
 
+        #update the simulation (evaluation of rhs)
+        self._update(t)
 
-    def _sample(self, t):
+
+    def _sample(self, t, dt):
         """Sample data from blocks that implement the 'sample' method such 
         as 'Scope', 'Delay' and the blocks that sample from a random 
         distribution at a given time 't'.
@@ -1151,24 +1150,34 @@ class Simulation:
             time where to sample
         """
         for block in self.blocks:
-            if block: block.sample(t)
+            if block: block.sample(t, dt)
 
 
-    def _buffer_blocks(self, dt):
-        """Buffer internal states of blocks before the timestep is taken. 
+    def _buffer(self, t, dt):
+        """Buffer states for event monitoring and internal states of blocks 
+        before the timestep is taken. 
 
-        This is required for runge-kutta integrators but also for the 
-        zero crossing detection of the event handling system.
-    
-        The timesteps are also buffered because some integrators such as 
-        GEAR-type methods need a history of the timesteps.
+        For events, this is required to set reference for event monitoring and 
+        backtracking for root finding.
+
+        for blocks, this is required for runge-kutta integrators but also for the 
+        zero crossing detection of the event handling system. The timesteps are 
+        also buffered because some integrators such as GEAR-type methods need a 
+        history of the timesteps.
 
         Parameters
         ----------
+        t : float 
+            evaluation time for buffering
         dt : float
             timestep
         """
-        #buffer the dummy engine
+
+        #buffer states for event detection (with timestamp)
+        for event in self._get_active_events():
+            event.buffer(t)
+
+        #buffer the dummy engine (allways)
         self.engine.buffer(dt)
 
         #buffer internal states of stateful blocks
@@ -1275,15 +1284,12 @@ class Simulation:
         if dt is None: 
             dt = self.dt
 
-        #buffer states for event system
-        self._buffer_events(self.time)
+        #buffer events and dynamic blocks
+        self._buffer(self.time, dt)
 
         #if no dynamic blocks -> skip the solver step        
         if self._blocks_dyn:
 
-            #buffer internal states for solvers
-            self._buffer_blocks(dt)
-
             #iterate explicit solver stages with evaluation time (generator)
             for time_stage in self.engine.stages(self.time, dt):
 
@@ -1312,7 +1318,7 @@ class Simulation:
             total_evals += 1
 
         #sample data after successful timestep (+dt)
-        self._sample(time_dt)
+        self._sample(time_dt, dt)
  
         #increment global time and continue simulation
         self.time = time_dt 
@@ -1354,15 +1360,12 @@ class Simulation:
         if dt is None: 
             dt = self.dt
 
-        #buffer states for event system
-        self._buffer_events(self.time)
+        #buffer events and dynamic blocks
+        self._buffer(self.time, dt)
 
         #if no dynamic blocks -> skip the solver step        
         if self._blocks_dyn:
 
-            #buffer internal states for solvers
-            self._buffer_blocks(dt)
-
             #iterate explicit solver stages with evaluation time (generator)
             for time_stage in self.engine.stages(self.time, dt):
 
@@ -1371,7 +1374,7 @@ class Simulation:
 
                 #warning if implicit solver didnt converge in timestep
                 if not success:
-                    self._logger_warning(
+                    self.logger.warning(
                         f"implicit solver not converged in {solver_its} iterations!"
                         )
 
@@ -1400,7 +1403,7 @@ class Simulation:
             total_evals += 1    
 
         #sample data after successful timestep (+dt)
-        self._sample(time_dt)
+        self._sample(time_dt, dt)
  
         #increment global time and continue simulation
         self.time = time_dt 
@@ -1449,15 +1452,12 @@ class Simulation:
         if dt is None: 
             dt = self.dt
 
-        #buffer states for event system
-        self._buffer_events(self.time)
+        #buffer events and dynamic blocks
+        self._buffer(self.time, dt)
 
         #if no dynamic blocks -> skip the solver step        
         if self._blocks_dyn:
 
-            #buffer internal states for solvers
-            self._buffer_blocks(dt)
-
             #iterate explicit solver stages with evaluation time (generator)
             for time_stage in self.engine.stages(self.time, dt):
 
@@ -1470,8 +1470,7 @@ class Simulation:
 
             #if step not successful -> roll back timestep
             if not success:
-                self._revert()
-                self._update(self.time) 
+                self._revert(self.time)
                 total_evals += 1
                 return False, error_norm, scale, total_evals, 0
 
@@ -1495,13 +1494,12 @@ class Simulation:
     
             #not close enough -> roll back timestep (secant step)
             else:
-                self._revert()
-                self._update(self.time) 
+                self._revert(self.time)
                 total_evals += 1
                 return False, error_norm, ratio, total_evals, 0
         
         #sample data after successful timestep (+dt)
-        self._sample(time_dt)
+        self._sample(time_dt, dt)
 
         #increment global time and continue simulation
         self.time = time_dt    
@@ -1554,15 +1552,12 @@ class Simulation:
         if dt is None: 
             dt = self.dt
 
-        #buffer states for event system
-        self._buffer_events(self.time)
+        #buffer events and dynamic blocks
+        self._buffer(self.time, dt)
 
         #if no dynamic blocks -> skip the solver step        
         if self._blocks_dyn:
 
-            #buffer internal states for solvers
-            self._buffer_blocks(dt)
-
             #iterate explicit solver stages with evaluation time (generator)
             for time_stage in self.engine.stages(self.time, dt):
 
@@ -1575,8 +1570,7 @@ class Simulation:
 
                 #if solver did not converge -> quit early (adaptive only)
                 if not success:
-                    self._revert()
-                    self._update(self.time) 
+                    self._revert(self.time)
                     return False, 0.0, 0.5, total_evals+1, total_solver_its  
 
                 #timestep for dynamical blocks (with internal states)
@@ -1584,8 +1578,7 @@ class Simulation:
 
             #if step not successful -> roll back timestep
             if not success:
-                self._revert()
-                self._update(self.time) 
+                self._revert(self.time)
                 return False, error_norm, scale, total_evals+1, total_solver_its
 
         #system time after timestep
@@ -1608,13 +1601,12 @@ class Simulation:
     
             #not close enough -> roll back timestep (secant step)
             else:
-                self._revert()
-                self._update(self.time) 
+                self._revert(self.time)
                 total_evals += 1
                 return False, error_norm, ratio, total_evals, total_solver_its
 
         #sample data after successful timestep (+dt)
-        self._sample(time_dt)
+        self._sample(time_dt, dt)
 
         #increment global time and continue simulation
         self.time = time_dt    
@@ -1663,7 +1655,7 @@ class Simulation:
 
     def step(self, dt=None, adaptive=True):
         """Wraps 'Simulation.timestep' for backward compatibility"""
-        self._logger_warning(
+        self.logger.warning(
             "'Simulation.step' method will be deprecated with release version 1.0.0, use 'Simulation.timestep' instead!"
             )
         return self.timestep(dt, adaptive)
@@ -1737,12 +1729,12 @@ class Simulation:
             initial_evals += 1
     
         #sampling states and inputs at 'self.time == starting_time' 
-        self._sample(self.time)
+        self._sample(self.time, self.dt)
 
         #initialize progress tracker
         tracker = ProgressTracker(
-            total_duration=duration, 
-            description="TRANSIENT", 
+            total_duration=duration,
+            description="TRANSIENT",
             logger=self.logger,
             log=self.log
             )
